# This file was automatically generated by SWIG (https://www.swig.org).
# Version 4.1.1
#
# Do not make changes to this file unless you know what you are doing - modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
import gphoto2.abilities_list
import gphoto2.context
import gphoto2.list
import gphoto2.port_info_list
import gphoto2.filesys
import gphoto2.file
import gphoto2.widget

# Pull in all the attributes from the low-level C/C++ module
if __package__ or "." in __name__:
    from ._camera import *
else:
    from _camera import *
